@@ -44,12 +44,22 @@ pub enum ExitStatus {
 type ControlFlow = std::ops::ControlFlow<ExitStatus>;
 
 #[derive(Parser, Debug)]
-#[command(version, about, name("numbat"), max_term_width = 90)]
+#[command(
+    version,
+    about,
+    name("numbat"),
+    max_term_width = 90,
+    trailing_var_arg = true
+)]
 struct Args {
     /// Path to source file with Numbat code. If none is given, an interactive
     /// session is started.
     file: Option<PathBuf>,
 
+    /// Command-line arguments passed to the Numbat program
+    #[arg(trailing_var_arg = true, allow_hyphen_values = true)]
+    script_args: Vec<String>,
+
     /// Evaluate a single expression. Can be specified multiple times to evaluate several expressions in sequence.
     #[arg(
         short,
@@ -526,11 +536,13 @@ impl Cli {
 
                 let ctx = self.context.lock().unwrap();
                 let registry = ctx.dimension_registry();
+                let format_options = self.config.formatting.to_format_options();
                 let result_markup = interpreter_result.to_markup(
                     statements.last(),
                     registry,
                     interactive || pretty_print,
                     interactive || pretty_print,
+                    &format_options,
                 );
                 print!("{}", ansi_format(&result_markup, false));
 
