@@ -5,7 +5,6 @@ use itertools::Itertools;
 
 use crate::ast::{ProcedureKind, TypeAnnotation};
 use crate::decorator::Decorator;
-use crate::dimension::DimensionRegistry;
 use crate::interpreter::{
     Interpreter, InterpreterResult, InterpreterSettings, Result, RuntimeError, RuntimeErrorKind,
 };
@@ -13,6 +12,7 @@ use crate::name_resolution::LAST_RESULT_IDENTIFIERS;
 use crate::prefix::Prefix;
 use crate::prefix_parser::AcceptsPrefix;
 use crate::pretty_print::PrettyPrint;
+use crate::typechecker::TypeChecker;
 use crate::typed_ast;
 use crate::typed_ast::{
     BinaryOperator, DefineVariable, Expression, Statement, StringPart, UnaryOperator,
@@ -20,7 +20,7 @@ use crate::typed_ast::{
 use crate::unit::{CanonicalName, Unit};
 use crate::unit_registry::{UnitMetadata, UnitRegistry};
 use crate::value::{FunctionReference, Value};
-use crate::vm::{Constant, ExecutionContext, FfiCallArg, Op, Vm};
+use crate::vm::{Constant, ExecutionContext, FfiCallArg, FfiCallArgs, Op, Vm};
 use crate::{Type, decorator};
 
 #[derive(Debug, Clone, Default)]
@@ -219,6 +219,7 @@ impl BytecodeInterpreter {
                 full_span,
                 name,
                 args,
+                type_scheme,
                 ..
             } => {
                 // Put all arguments on top of the stack
@@ -228,13 +229,16 @@ impl BytecodeInterpreter {
 
                 if let Some(idx) = self.vm.get_ffi_callable_idx(name) {
                     // TODO: check overflow:
-                    let call_args: Vec<FfiCallArg> = args
-                        .iter()
-                        .map(|a: &typed_ast::Expression| FfiCallArg {
-                            span: a.full_span(),
-                            type_: a.get_type_scheme(),
-                        })
-                        .collect();
+                    let call_args = FfiCallArgs {
+                        args: args
+                            .iter()
+                            .map(|a: &typed_ast::Expression| FfiCallArg {
+                                span: a.full_span(),
+                                type_: a.get_type_scheme(),
+                            })
+                            .collect(),
+                        return_type: Some(type_scheme.clone()),
+                    };
                     let call_args_idx = self.vm.add_ffi_call_args(call_args);
                     self.vm.add_op3(
                         Op::FFICallFunction,
@@ -300,7 +304,7 @@ impl BytecodeInterpreter {
                 full_span,
                 callable,
                 args,
-                ..
+                type_scheme,
             } => {
                 // Put all arguments on top of the stack
                 for arg in args {
@@ -310,13 +314,16 @@ impl BytecodeInterpreter {
                 // Put the callable on top of the stack
                 self.compile_expression(callable);
 
-                let call_args: Vec<FfiCallArg> = args
-                    .iter()
-                    .map(|a| FfiCallArg {
-                        span: a.full_span(),
-                        type_: a.get_type_scheme(),
-                    })
-                    .collect();
+                let call_args = FfiCallArgs {
+                    args: args
+                        .iter()
+                        .map(|a| FfiCallArg {
+                            span: a.full_span(),
+                            type_: a.get_type_scheme(),
+                        })
+                        .collect(),
+                    return_type: Some(type_scheme.clone()),
+                };
                 let call_args_idx = self.vm.add_ffi_call_args(call_args);
                 self.vm.add_op2(
                     Op::CallCallable,
@@ -419,11 +426,7 @@ impl BytecodeInterpreter {
         });
     }
 
-    fn compile_statement(
-        &mut self,
-        stmt: &Statement,
-        dimension_registry: &DimensionRegistry,
-    ) -> Result<()> {
+    fn compile_statement(&mut self, stmt: &Statement, typechecker: &TypeChecker) -> Result<()> {
         match stmt {
             Statement::Expression(expr) => {
                 self.compile_expression(expr);
@@ -502,7 +505,9 @@ impl BytecodeInterpreter {
                             readable_type: type_annotation
                                 .as_ref()
                                 .map(|a: &TypeAnnotation| a.pretty_print())
-                                .unwrap_or(type_scheme.to_readable_type(dimension_registry, false)),
+                                .unwrap_or(
+                                    type_scheme.to_readable_type(typechecker.registry(), false),
+                                ),
                             aliases,
                             name: decorator::name(decorators).map(CompactString::from),
                             canonical_name: decorator::get_canonical_unit_name(
@@ -512,6 +517,7 @@ impl BytecodeInterpreter {
                             description: decorator::description(decorators),
                             binary_prefixes: decorators.contains(&Decorator::BinaryPrefixes),
                             metric_prefixes: decorators.contains(&Decorator::MetricPrefixes),
+                            is_abbreviation: decorator::contains_abbreviation(decorators),
                             code_source_id: span.code_source_id,
                         },
                     )
@@ -556,7 +562,7 @@ impl BytecodeInterpreter {
                         readable_type: type_annotation
                             .as_ref()
                             .map(|a: &TypeAnnotation| a.pretty_print())
-                            .unwrap_or(type_scheme.to_readable_type(dimension_registry, false)),
+                            .unwrap_or(type_scheme.to_readable_type(typechecker.registry(), false)),
                         aliases,
                         name: decorator::name(decorators).map(CompactString::from),
                         canonical_name: decorator::get_canonical_unit_name(unit_name, decorators),
@@ -564,6 +570,7 @@ impl BytecodeInterpreter {
                         description: decorator::description(decorators),
                         binary_prefixes: decorators.contains(&Decorator::BinaryPrefixes),
                         metric_prefixes: decorators.contains(&Decorator::MetricPrefixes),
+                        is_abbreviation: decorator::contains_abbreviation(decorators),
                         code_source_id: full_span.code_source_id,
                     },
                 ); // TODO: there is some asymmetry here because we do not introduce identifiers for base units
@@ -610,13 +617,16 @@ impl BytecodeInterpreter {
 
                 let callable_idx = self.vm.get_ffi_callable_idx(name).unwrap();
 
-                let call_args: Vec<FfiCallArg> = args
-                    .iter()
-                    .map(|a: &typed_ast::Expression| FfiCallArg {
-                        span: a.full_span(),
-                        type_: a.get_type_scheme(),
-                    })
-                    .collect();
+                let call_args = FfiCallArgs {
+                    args: args
+                        .iter()
+                        .map(|a: &typed_ast::Expression| FfiCallArg {
+                            span: a.full_span(),
+                            type_: a.get_type_scheme(),
+                        })
+                        .collect(),
+                    return_type: None,
+                };
                 let call_args_idx = self.vm.add_ffi_call_args(call_args);
 
                 self.vm.add_op3(
@@ -639,11 +649,14 @@ impl BytecodeInterpreter {
     fn run(
         &mut self,
         settings: &mut InterpreterSettings,
-        dimension_registry: &DimensionRegistry,
+        prefix_transformer: &crate::prefix_transformer::Transformer,
+        typechecker: &TypeChecker,
     ) -> Result<InterpreterResult> {
         let mut ctx = ExecutionContext {
             print_fn: &mut settings.print_fn,
-            dimension_registry,
+            unit_name_to_constant_idx: &self.unit_name_to_constant_index,
+            prefix_transformer,
+            typechecker,
         };
 
         self.vm.disassemble();
@@ -652,7 +665,10 @@ impl BytecodeInterpreter {
 
         let result = match result {
             Ok(InterpreterResult::Value(Value::Quantity(q))) => {
-                Ok(InterpreterResult::Value(Value::Quantity(q.full_simplify())))
+                let simplified = self
+                    .vm
+                    .simplify_quantity(&q, &self.unit_name_to_constant_index);
+                Ok(InterpreterResult::Value(Value::Quantity(simplified)))
             }
             r => r,
         };
@@ -701,13 +717,14 @@ impl Interpreter for BytecodeInterpreter {
         &mut self,
         settings: &mut InterpreterSettings,
         statements: &[Statement],
-        dimension_registry: &DimensionRegistry,
+        prefix_transformer: &crate::prefix_transformer::Transformer,
+        typechecker: &TypeChecker,
     ) -> Result<InterpreterResult> {
         for statement in statements {
-            self.compile_statement(statement, dimension_registry)?;
+            self.compile_statement(statement, typechecker)?;
         }
 
-        self.run(settings, dimension_registry)
+        self.run(settings, prefix_transformer, typechecker)
     }
 
     fn get_unit_registry(&self) -> &UnitRegistry {
