@@ -6,6 +6,7 @@ use crate::{prefix_parser::AcceptsPrefix, span::Span, unit::CanonicalName};
 pub enum Decorator<'a> {
     MetricPrefixes,
     BinaryPrefixes,
+    Abbreviation,
     Aliases(Vec<(&'a str, Option<AcceptsPrefix>, Span)>),
     Url(CompactString),
     Name(CompactString),
@@ -161,3 +162,13 @@ pub fn contains_examples(decorators: &[Decorator]) -> bool {
 
     false
 }
+
+pub fn contains_abbreviation(decorators: &[Decorator]) -> bool {
+    for decorator in decorators {
+        if let Decorator::Abbreviation = decorator {
+            return true;
+        }
+    }
+
+    false
+}
