@@ -7,22 +7,28 @@ use jiff::fmt::strtime::BrokenDownTime;
 use num_traits::ToPrimitive;
 
 use super::Args;
+use super::FfiContext;
 use super::Result;
 use super::macros::*;
 use crate::datetime;
 use crate::interpreter::RuntimeErrorKind;
 use crate::quantity::Quantity;
+use crate::typechecker::type_scheme::TypeScheme;
 use crate::value::FunctionReference;
 use crate::value::Value;
-use crate::vm::ExecutionContext;
 
-pub fn now(_ctx: &mut ExecutionContext, _args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn now(
+    _ctx: &mut FfiContext,
+    _args: Args,
+    _return_type: &TypeScheme,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     return_datetime!(Zoned::now())
 }
 
 pub fn datetime(
-    _ctx: &mut ExecutionContext,
+    _ctx: &mut FfiContext,
     mut args: Args,
+    _return_type: &TypeScheme,
 ) -> Result<Value, Box<RuntimeErrorKind>> {
     let input = string_arg!(args);
 
@@ -33,8 +39,9 @@ pub fn datetime(
 }
 
 pub fn format_datetime(
-    _ctx: &mut ExecutionContext,
+    _ctx: &mut FfiContext,
     mut args: Args,
+    _return_type: &TypeScheme,
 ) -> Result<Value, Box<RuntimeErrorKind>> {
     let format = string_arg!(args);
     let dt = datetime_arg!(args);
@@ -51,8 +58,9 @@ pub fn format_datetime(
 }
 
 pub fn get_local_timezone(
-    _ctx: &mut ExecutionContext,
+    _ctx: &mut FfiContext,
     _args: Args,
+    _return_type: &TypeScheme,
 ) -> Result<Value, Box<RuntimeErrorKind>> {
     let local_tz = datetime::get_local_timezone_or_utc();
     let tz_name = local_tz.iana_name().unwrap_or("<unknown timezone>");
@@ -60,7 +68,11 @@ pub fn get_local_timezone(
     return_string!(borrowed = tz_name)
 }
 
-pub fn tz(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn tz(
+    _ctx: &mut FfiContext,
+    mut args: Args,
+    _return_type: &TypeScheme,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     let tz = string_arg!(args);
 
     Ok(Value::FunctionReference(FunctionReference::TzConversion(
@@ -68,24 +80,26 @@ pub fn tz(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<Runt
     )))
 }
 
-pub fn unixtime(
-    _ctx: &mut ExecutionContext,
+pub fn unixtime_us(
+    _ctx: &mut FfiContext,
     mut args: Args,
+    _return_type: &TypeScheme,
 ) -> Result<Value, Box<RuntimeErrorKind>> {
     let input = datetime_arg!(args);
 
-    let output = input.timestamp().as_second();
+    let us = input.timestamp().as_microsecond();
 
-    return_scalar!(output as f64)
+    return_scalar!(us as f64)
 }
 
-pub fn from_unixtime(
-    _ctx: &mut ExecutionContext,
+pub fn from_unixtime_us(
+    _ctx: &mut FfiContext,
     mut args: Args,
+    _return_type: &TypeScheme,
 ) -> Result<Value, Box<RuntimeErrorKind>> {
-    let timestamp = quantity_arg!(args).unsafe_value().to_f64() as i64;
+    let us = quantity_arg!(args).unsafe_value().to_f64() as i64;
 
-    let dt = Timestamp::from_second(timestamp)
+    let dt = Timestamp::from_microsecond(us)
         .map_err(|_| RuntimeErrorKind::DateTimeOutOfRange)?
         .to_zoned(datetime::get_local_timezone_or_utc());
 
@@ -117,17 +131,26 @@ fn calendar_add(
     return_datetime!(output)
 }
 
-pub fn _add_days(_ctx: &mut ExecutionContext, args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn _add_days(
+    _ctx: &mut FfiContext,
+    args: Args,
+    _return_type: &TypeScheme,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     calendar_add(args, "day", |n| Span::new().try_days(n))
 }
 
 pub fn _add_months(
-    _ctx: &mut ExecutionContext,
+    _ctx: &mut FfiContext,
     args: Args,
+    _return_type: &TypeScheme,
 ) -> Result<Value, Box<RuntimeErrorKind>> {
     calendar_add(args, "month", |n| Span::new().try_months(n))
 }
 
-pub fn _add_years(_ctx: &mut ExecutionContext, args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn _add_years(
+    _ctx: &mut FfiContext,
+    args: Args,
+    _return_type: &TypeScheme,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     calendar_add(args, "year", |n| Span::new().try_years(n))
 }
