@@ -1,17 +1,25 @@
 use super::macros::*;
-use super::{Args, Result};
+use super::{Args, FfiContext, Result};
 use crate::interpreter::RuntimeErrorKind;
 use crate::quantity::Quantity;
+use crate::typechecker::type_scheme::TypeScheme;
 use crate::value::Value;
-use crate::vm::ExecutionContext;
 
-pub fn len(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn len(
+    _ctx: &mut FfiContext,
+    mut args: Args,
+    _return_type: &TypeScheme,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     let list = list_arg!(args);
 
     return_scalar!(list.len() as f64)
 }
 
-pub fn head(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn head(
+    _ctx: &mut FfiContext,
+    mut args: Args,
+    _return_type: &TypeScheme,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     let list = list_arg!(args);
 
     if let Some(first) = list.head() {
@@ -21,14 +29,22 @@ pub fn head(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<Ru
     }
 }
 
-pub fn tail(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn tail(
+    _ctx: &mut FfiContext,
+    mut args: Args,
+    _return_type: &TypeScheme,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     let mut list = list_arg!(args);
 
     list.tail()?;
     Ok(list.into())
 }
 
-pub fn cons(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<RuntimeErrorKind>> {
+pub fn cons(
+    _ctx: &mut FfiContext,
+    mut args: Args,
+    _return_type: &TypeScheme,
+) -> Result<Value, Box<RuntimeErrorKind>> {
     let element = arg!(args);
     let mut list = list_arg!(args);
     list.push_front(element);
@@ -37,8 +53,9 @@ pub fn cons(_ctx: &mut ExecutionContext, mut args: Args) -> Result<Value, Box<Ru
 }
 
 pub fn cons_end(
-    _ctx: &mut ExecutionContext,
+    _ctx: &mut FfiContext,
     mut args: Args,
+    _return_type: &TypeScheme,
 ) -> Result<Value, Box<RuntimeErrorKind>> {
     let element = arg!(args);
     let mut list = list_arg!(args);
