@@ -5,9 +5,11 @@ use core::fmt;
 use crate::{
     dimension::DimensionRegistry,
     markup::Markup,
-    pretty_print::PrettyPrint,
+    prefix_transformer::Transformer,
+    pretty_print::FormatOptions,
     quantity::QuantityError,
     span::Span,
+    typechecker::TypeChecker,
     typed_ast::Statement,
     unit_registry::{UnitRegistry, UnitRegistryError},
 };
@@ -102,6 +104,7 @@ impl InterpreterResult {
         registry: &DimensionRegistry,
         with_type_info: bool,
         with_equal_sign: bool,
+        format_options: &FormatOptions,
     ) -> Markup {
         match self {
             Self::Value(value) => {
@@ -128,7 +131,7 @@ impl InterpreterResult {
                     m::empty()
                 };
 
-                leader + value.pretty_print() + type_markup + m::nl()
+                leader + value.pretty_print_with(format_options) + type_markup + m::nl()
             }
             Self::Continue => m::empty(),
         }
@@ -183,7 +186,8 @@ pub trait Interpreter {
         &mut self,
         settings: &mut InterpreterSettings,
         statements: &[Statement],
-        dimension_registry: &DimensionRegistry,
+        prefix_transformer: &Transformer,
+        typechecker: &TypeChecker,
     ) -> Result<InterpreterResult>;
     fn get_unit_registry(&self) -> &UnitRegistry;
 }
@@ -230,17 +234,19 @@ mod tests {
         let full_code = format!("{TEST_PRELUDE}\n{input}");
         let statements = crate::parser::parse(&full_code, 0)
             .expect("No parse errors for inputs in this test suite");
-        let statements_transformed = Transformer::new()
+        let mut transformer = Transformer::new();
+        let statements_transformed = transformer
             .transform(statements)
             .expect("No name resolution errors for inputs in this test suite");
-        let mut typechecker = crate::typechecker::TypeChecker::default();
+        let mut typechecker = TypeChecker::default();
         let statements_typechecked = typechecker
             .check(&statements_transformed)
             .expect("No type check errors for inputs in this test suite");
         BytecodeInterpreter::new().interpret_statements(
             &mut InterpreterSettings::default(),
             &statements_typechecked,
-            typechecker.registry(),
+            &transformer,
+            &typechecker,
         )
     }
 
