@@ -6,7 +6,7 @@ mod bytecode_interpreter;
 mod column_formatter;
 pub mod command;
 mod currency;
-mod datetime;
+pub mod datetime;
 mod decorator;
 pub mod diagnostic;
 mod dimension;
@@ -23,6 +23,7 @@ mod math;
 pub mod module_importer;
 mod name_resolution;
 mod number;
+mod parse_quantity;
 mod parser;
 #[cfg(feature = "plotting")]
 mod plot;
@@ -71,7 +72,7 @@ use resolver::CodeSource;
 use resolver::Resolver;
 use resolver::ResolverError;
 use thiserror::Error;
-use typechecker::{TypeCheckError, TypeChecker};
+use typechecker::TypeChecker;
 
 pub use diagnostic::Diagnostic;
 pub use interpreter::InterpreterResult;
@@ -79,8 +80,10 @@ pub use interpreter::InterpreterSettings;
 pub use interpreter::{RuntimeError, RuntimeErrorKind};
 pub use name_resolution::NameResolutionError;
 pub use parser::ParseError;
+pub use pretty_print::FormatOptions;
 pub use registry::BaseRepresentation;
 pub use registry::BaseRepresentationFactor;
+pub use typechecker::TypeCheckError;
 pub use typed_ast::Statement;
 pub use typed_ast::Type;
 use unit::BaseUnitAndFactor;
@@ -297,9 +300,20 @@ impl Context {
             add_if_valid(variable.into());
         }
 
+        // Temperature conversion functions that are used as pseudo-units (e.g., "20 celsius")
+        // should not have parentheses added during completion
+        const TEMPERATURE_PSEUDO_UNITS: &[&str] = &[
+            "celsius",
+            "degree_celsius",
+            "fahrenheit",
+            "degree_fahrenheit",
+            "°C",
+            "°F",
+        ];
+
         for function_name in self.function_names() {
             let mut function = function_name.clone();
-            if add_paren {
+            if add_paren && !TEMPERATURE_PSEUDO_UNITS.contains(&function_name.as_str()) {
                 if let Some((signature, _)) = self.typechecker.lookup_function(&function_name) {
                     if signature.parameters.is_empty() {
                         function.push_str("()");
@@ -576,7 +590,14 @@ impl Context {
             }
 
             if let Ok((_, results)) = self.interpret(keyword, CodeSource::Internal) {
-                help += m::nl() + results.to_markup(None, self.dimension_registry(), true, true);
+                help += m::nl()
+                    + results.to_markup(
+                        None,
+                        self.dimension_registry(),
+                        true,
+                        true,
+                        &FormatOptions::default(),
+                    );
             }
 
             return help;
@@ -789,7 +810,8 @@ impl Context {
         let result = self.interpreter.interpret_statements(
             settings,
             &typed_statements,
-            self.typechecker.registry(),
+            &self.prefix_transformer,
+            &self.typechecker,
         );
 
         if result.is_err() {
