@@ -10,6 +10,16 @@ use crate::{
 
 type Result<T> = std::result::Result<T, NameResolutionError>;
 
+/// Check if the identifier is a temperature pseudo-unit (°C, celsius, etc.)
+/// and return the corresponding conversion function name.
+fn temperature_conversion_function(ident: &str) -> Option<&'static str> {
+    match ident {
+        "°C" | "celsius" | "degree_celsius" => Some("from_celsius"),
+        "°F" | "fahrenheit" | "degree_fahrenheit" => Some("from_fahrenheit"),
+        _ => None,
+    }
+}
+
 #[derive(Debug, Clone)]
 pub(crate) struct Transformer {
     pub prefix_parser: PrefixParser,
@@ -31,7 +41,7 @@ impl Transformer {
         }
     }
 
-    fn transform_expression(&self, expression: &mut Expression) {
+    pub fn transform_expression(&self, expression: &mut Expression) {
         match expression {
             Expression::Scalar(..) | Expression::Boolean(_, _) | Expression::TypedHole(_) => {}
             Expression::Identifier(span, identifier) => {
@@ -55,11 +65,65 @@ impl Transformer {
             Expression::UnitIdentifier { .. } => {
                 unreachable!("Prefixed identifiers should not exist prior to this stage")
             }
-            Expression::UnaryOperator { expr, .. } => self.transform_expression(expr),
+            Expression::UnaryOperator { op, expr, span_op } => {
+                // Syntactic sugar for entering temperatures in °C or °F. Transform
+                // `-5 °C`, which is parsed as -(5 °C), into from_celsius(-5).
+                if *op == crate::ast::UnaryOperator::Negate
+                    && let Expression::BinaryOperator {
+                        op: bin_op,
+                        lhs: inner_lhs,
+                        rhs: inner_rhs,
+                        span_op: _,
+                    } = expr.as_mut()
+                    && *bin_op == crate::ast::BinaryOperator::Mul
+                    && let Expression::Identifier(rhs_span, ident) = inner_rhs.as_ref()
+                    && let Some(fn_name) = temperature_conversion_function(ident)
+                {
+                    // Transform the inner lhs first
+                    self.transform_expression(inner_lhs);
 
-            Expression::BinaryOperator { lhs, rhs, .. } => {
+                    // Create negated argument: -<inner_lhs>
+                    let negated_arg = Expression::UnaryOperator {
+                        op: crate::ast::UnaryOperator::Negate,
+                        expr: Box::new((**inner_lhs).clone()),
+                        span_op: *span_op,
+                    };
+
+                    let full_span = span_op.extend(rhs_span);
+                    *expression = Expression::FunctionCall {
+                        ident_span: *rhs_span,
+                        full_span,
+                        callable: Box::new(Expression::Identifier(*rhs_span, fn_name)),
+                        args: vec![negated_arg],
+                    };
+                    return;
+                }
+                self.transform_expression(expr);
+            }
+
+            Expression::BinaryOperator {
+                op,
+                lhs,
+                rhs,
+                span_op: _,
+            } => {
                 self.transform_expression(lhs);
                 self.transform_expression(rhs);
+
+                // Syntactic sugar for entering temperatures in °C or °F. Transform `5 °C`,
+                // which is parsed as 5 * °C, into from_celsius(5).
+                if *op == crate::ast::BinaryOperator::Mul
+                    && let Expression::Identifier(rhs_span, ident) = rhs.as_ref()
+                    && let Some(fn_name) = temperature_conversion_function(ident)
+                {
+                    let full_span = lhs.full_span().extend(rhs_span);
+                    *expression = Expression::FunctionCall {
+                        ident_span: *rhs_span,
+                        full_span,
+                        callable: Box::new(Expression::Identifier(*rhs_span, fn_name)),
+                        args: vec![(**lhs).clone()],
+                    };
+                }
             }
             Expression::FunctionCall { args, .. } => {
                 for arg in args {
@@ -137,7 +201,11 @@ impl Transformer {
         Ok(())
     }
 
-    fn transform_define_variable(&mut self, define_variable: &mut DefineVariable) -> Result<()> {
+    fn transform_define_variable(
+        &mut self,
+        define_variable: &mut DefineVariable,
+        allow_shadowing: bool,
+    ) -> Result<()> {
         let DefineVariable {
             identifier_span,
             identifier,
@@ -149,8 +217,13 @@ impl Transformer {
         for (name, _) in decorator::name_and_aliases(identifier, decorators) {
             self.variable_names.push(name.to_compact_string());
         }
-        self.prefix_parser
-            .add_other_identifier(identifier, *identifier_span)?;
+        if allow_shadowing {
+            self.prefix_parser
+                .add_shadowing_identifier(identifier, *identifier_span)?;
+        } else {
+            self.prefix_parser
+                .add_other_identifier(identifier, *identifier_span)?;
+        }
         self.transform_expression(expr);
 
         Ok(())
@@ -177,7 +250,8 @@ impl Transformer {
                 self.transform_expression(expr);
             }
             Statement::DefineVariable(define_variable) => {
-                self.transform_define_variable(define_variable)?
+                // Global variables should not shadow unit names
+                self.transform_define_variable(define_variable, false)?
             }
             Statement::DefineFunction {
                 function_name_span,
@@ -194,24 +268,37 @@ impl Transformer {
                 // We create a clone of the full transformer for the purpose
                 // of checking/transforming the function body. The reason for this
                 // is that we don't want the parameter names to pollute the global
-                // namespace. But we need to register parameter names as identifiers
-                // because they could otherwise shadow global identifiers:
+                // namespace. Function parameters and local variables (from where
+                // clauses) are allowed to shadow global definitions (including
+                // unit names), e.g.:
                 //
-                //   fn foo(t: Time) -> Time = t    # not okay: shadows 't' for ton
+                //   fn foo(h: Length) -> Length = 2 h    # okay: 'h' shadows the hour unit
                 //
                 let mut fn_body_transformer = self.clone();
                 for (param_span, param, _) in &*parameters {
                     fn_body_transformer
                         .prefix_parser
-                        .add_other_identifier(param, *param_span)?;
+                        .add_shadowing_identifier(param, *param_span)?;
+                }
+
+                // Register local variable identifiers before transforming the body,
+                // since the body expression may reference them (where clauses).
+                for def in &mut *local_variables {
+                    fn_body_transformer
+                        .variable_names
+                        .push(def.identifier.to_compact_string());
+                    fn_body_transformer
+                        .prefix_parser
+                        .add_shadowing_identifier(def.identifier, def.identifier_span)?;
                 }
 
                 if let Some(expr) = body {
                     fn_body_transformer.transform_expression(expr);
                 }
 
+                // Now transform the local variable expressions
                 for def in local_variables {
-                    fn_body_transformer.transform_define_variable(def)?;
+                    fn_body_transformer.transform_expression(&mut def.expr);
                 }
             }
             Statement::DefineDimension(_, name, _) => {
