@@ -1,7 +1,8 @@
 use crate::arithmetic::{Exponent, Power, Rational};
 use crate::number::Number;
-use crate::pretty_print::PrettyPrint;
+use crate::pretty_print::FormatOptions;
 use crate::unit::{Unit, UnitFactor, is_multiple_of};
+use crate::unit_registry::UnitRegistry;
 
 use compact_str::{CompactString, ToCompactString, format_compact};
 use itertools::Itertools;
@@ -26,6 +27,10 @@ pub struct Quantity {
     value: Number,
     unit: Unit,
     can_simplify: bool,
+    /// If set, the quantity is displayed as `(value/target) × target`
+    /// instead of `value unit`. This is used for conversions like `6 hours -> 45 min`
+    /// which should display as `8 × 45 min` instead of `360 min`.
+    conversion_target: Option<Box<Quantity>>,
 }
 
 impl Quantity {
@@ -34,6 +39,7 @@ impl Quantity {
             value,
             unit,
             can_simplify: true,
+            conversion_target: None,
         }
     }
 
@@ -42,6 +48,7 @@ impl Quantity {
             value: Number::from_f64(value),
             unit,
             can_simplify: true,
+            conversion_target: None,
         }
     }
 
@@ -50,6 +57,14 @@ impl Quantity {
         self
     }
 
+    pub fn with_conversion_target(mut self, target: Quantity) -> Self {
+        // Only set the target if its value is not 1
+        if target.value.to_f64() != 1.0 {
+            self.conversion_target = Some(Box::new(target));
+        }
+        self
+    }
+
     pub fn from_scalar(value: f64) -> Quantity {
         Quantity::new_f64(value, Unit::scalar())
     }
@@ -62,6 +77,10 @@ impl Quantity {
         &self.unit
     }
 
+    pub fn can_simplify(&self) -> bool {
+        self.can_simplify
+    }
+
     pub fn is_zero(&self) -> bool {
         self.value.to_f64() == 0.0
     }
@@ -231,6 +250,88 @@ impl Quantity {
         Quantity::new(self.value * factor, simplified_unit)
     }
 
+    /// Like `full_simplify`, but also tries to simplify to registered derived units.
+    /// For example, J/s -> W, Pa·m² -> N, etc.
+    ///
+    /// The `get_unit` closure should return the Unit for a given unit name,
+    /// typically by looking it up in the VM's constants.
+    pub fn full_simplify_with_registry<F>(&self, registry: &UnitRegistry, get_unit: F) -> Self
+    where
+        F: Fn(&str) -> Option<Unit>,
+    {
+        // First apply the standard simplification
+        let simplified = self.full_simplify();
+
+        if !simplified.can_simplify {
+            return simplified;
+        }
+
+        let current_complexity = simplified.unit.iter().count();
+
+        // Only try registry lookup if we have more than one factor
+        if current_complexity <= 1 {
+            return simplified;
+        }
+
+        let (base_unit_repr, source_factor) = simplified.unit.to_base_unit_representation();
+
+        // Check if the base representation is a single base unit with an integer exponent,
+        // and the conversion factor is 1 (i.e., the unit is definitionally equivalent).
+        // For example, N/Pa has base repr m² with factor 1, so we simplify to m².
+        // But Wh/W has base repr s with factor 3600, so we don't simplify to s.
+        if base_unit_repr.iter().count() == 1
+            && let Some(factor) = base_unit_repr.iter().next()
+            && factor.exponent.is_integer()
+            && (source_factor.to_f64() - 1.0).abs() < 1e-9
+            && let Ok(converted) = simplified.convert_to(&base_unit_repr)
+        {
+            return converted;
+        }
+
+        // Look up matching unit names from the registry
+        let matching_names = registry.get_matching_unit_names(&simplified.unit);
+
+        // Try each matching unit and pick the simplest one we can convert to
+        let mut best: Option<Quantity> = None;
+
+        for name in matching_names {
+            if let Some(target_unit) = get_unit(&name) {
+                let target_complexity = target_unit.iter().count();
+
+                // Only consider if it's simpler
+                if target_complexity < current_complexity {
+                    // Check if the source unit and target unit have the same
+                    // conversion factor to base units. This ensures we only
+                    // simplify when the units are definitionally equivalent
+                    // (e.g., J/s = W) and not just dimensionally equivalent
+                    // (e.g., Wh/W ≠ century, even though both are time).
+                    let (_, target_factor) = target_unit.to_base_unit_representation();
+
+                    // Only simplify if conversion factors are equal (within tolerance)
+                    let factors_match = (source_factor.to_f64() - target_factor.to_f64()).abs()
+                        < 1e-9 * source_factor.to_f64().abs().max(1.0);
+
+                    if factors_match && let Ok(converted) = simplified.convert_to(&target_unit) {
+                        // Can't get simpler than 1 factor, return immediately
+                        if target_complexity == 1 {
+                            return converted;
+                        }
+
+                        let dominated = match &best {
+                            None => true,
+                            Some(best_q) => target_complexity < best_q.unit.iter().count(),
+                        };
+                        if dominated {
+                            best = Some(converted);
+                        }
+                    }
+                }
+            }
+        }
+
+        best.unwrap_or(simplified)
+    }
+
     pub fn as_scalar(&self) -> Result<Number> {
         Ok(self.convert_to(&Unit::scalar())?.value)
     }
@@ -277,10 +378,14 @@ impl std::ops::Add for &Quantity {
             Ok(rhs.clone())
         } else if rhs.is_zero() {
             Ok(self.clone())
+        } else if self.unit == rhs.unit {
+            Ok(Quantity::new(self.value + rhs.value, self.unit.clone()))
         } else {
+            // Use the smaller unit to ensure commutativity: a + b == b + a
+            let result_unit = self.unit.smaller_unit(&rhs.unit);
             Ok(Quantity::new(
-                self.value + rhs.convert_to(&self.unit)?.value,
-                self.unit.clone(),
+                self.convert_to(result_unit)?.value + rhs.convert_to(result_unit)?.value,
+                result_unit.clone(),
             ))
         }
     }
@@ -294,10 +399,14 @@ impl std::ops::Sub for &Quantity {
             Ok(-rhs.clone())
         } else if rhs.is_zero() {
             Ok(self.clone())
+        } else if self.unit == rhs.unit {
+            Ok(Quantity::new(self.value - rhs.value, self.unit.clone()))
         } else {
+            // Use the smaller unit to ensure anticommutativity: a - b == -(b - a)
+            let result_unit = self.unit.smaller_unit(&rhs.unit);
             Ok(Quantity::new(
-                self.value - rhs.convert_to(&self.unit)?.value,
-                self.unit.clone(),
+                self.convert_to(result_unit)?.value - rhs.convert_to(result_unit)?.value,
+                result_unit.clone(),
             ))
         }
     }
@@ -344,9 +453,13 @@ impl PartialOrd for Quantity {
     }
 }
 
-impl PrettyPrint for Quantity {
-    fn pretty_print(&self) -> crate::markup::Markup {
-        self.pretty_print_with_options(None)
+impl Quantity {
+    pub fn pretty_print(&self) -> crate::markup::Markup {
+        self.pretty_print_with(&FormatOptions::default())
+    }
+
+    pub fn pretty_print_with(&self, format_options: &FormatOptions) -> crate::markup::Markup {
+        self.pretty_print_internal(format_options, None)
     }
 }
 
@@ -376,15 +489,35 @@ impl Quantity {
         QuantityOrdering::Ok(cmp)
     }
 
-    /// Pretty prints with the given options.
-    /// If options is None, default options will be used.
-    fn pretty_print_with_options(&self, options: Option<FmtFloatConfig>) -> crate::markup::Markup {
+    /// Pretty prints with the given format options and optional dtoa config override.
+    fn pretty_print_internal(
+        &self,
+        format_options: &FormatOptions,
+        dtoa_config: Option<FmtFloatConfig>,
+    ) -> crate::markup::Markup {
         use crate::markup;
 
-        let formatted_number = self.unsafe_value().pretty_print_with_options(options);
-
         let unit_str = format_compact!("{}", self.unit());
 
+        // If there's a conversion target, display as `coefficient × target`
+        // e.g., `6 hours -> 45 min` displays as `8 × 45 min`
+        if let Some(ref target) = self.conversion_target {
+            let coefficient = self.value / target.value;
+            let formatted_coefficient =
+                coefficient.pretty_print_with_dtoa_config(format_options, dtoa_config);
+            let formatted_target = target.pretty_print_internal(format_options, dtoa_config);
+
+            return markup::value(formatted_coefficient)
+                + markup::space()
+                + markup::operator("×")
+                + markup::space()
+                + formatted_target;
+        }
+
+        let formatted_number = self
+            .unsafe_value()
+            .pretty_print_with_dtoa_config(format_options, dtoa_config);
+
         markup::value(formatted_number)
             + if unit_str == "°" || unit_str == "′" || unit_str == "″" || unit_str.is_empty() {
                 markup::empty()
@@ -397,13 +530,13 @@ impl Quantity {
     /// Pretty prints with the given precision. Disables e (scientific) notation.
     /// Prints without fractional part if precision is 0.
     pub fn pretty_print_with_precision(&self, precision: i8) -> crate::markup::Markup {
-        let options = FmtFloatConfig::default()
+        let dtoa_config = FmtFloatConfig::default()
             .min_decimal_digits(precision)
             .max_decimal_digits(precision)
             .add_point_zero(false)
             .force_no_e_notation()
             .round();
-        self.pretty_print_with_options(Some(options))
+        self.pretty_print_internal(&FormatOptions::default(), Some(dtoa_config))
     }
 
     pub fn unsafe_value_as_string(&self) -> CompactString {
