@@ -117,6 +117,22 @@ impl<Metadata: Clone> Registry<Metadata> {
             .collect()
     }
 
+    pub fn get_derived_entry_names_for_filtered<F>(
+        &self,
+        base_representation: &BaseRepresentation,
+        filter: F,
+    ) -> Vec<CompactString>
+    where
+        F: Fn(&Metadata) -> bool,
+    {
+        self.derived_entries
+            .iter()
+            .filter(|(_, (br, metadata))| br == base_representation && filter(metadata))
+            .map(|(name, _)| name.clone())
+            .sorted_unstable()
+            .collect()
+    }
+
     pub fn add_derived_entry(
         &mut self,
         name: &str,
@@ -184,4 +200,8 @@ impl<Metadata: Clone> Registry<Metadata> {
             .iter()
             .any(|(name, _)| name == dimension_name)
     }
+
+    pub fn is_base_unit(&self, unit_name: &str) -> bool {
+        self.base_entries.iter().any(|(name, _)| name == unit_name)
+    }
 }
