@@ -1,3 +1,5 @@
+use compact_str::ToCompactString;
+
 use super::name_generator::NameGenerator;
 use super::qualified_type::{Bound, Bounds, QualifiedType};
 use super::substitutions::{ApplySubstitution, Substitution, SubstitutionError};
@@ -6,15 +8,18 @@ use crate::pretty_print::PrettyPrint;
 use crate::type_variable::TypeVariable;
 use crate::typed_ast::Type;
 
-/// A type *scheme* (the `Quantified` part below) is a type with a
-/// set of universally quantified type variables.
+/// A (possibly polymorphic) type with universally quantified type variables.
 ///
-/// The way this is represented is by using type variables of kind
-/// `Quantified(i)` in the type tree, where `i` is a unique index
-/// for each universally quantified type variable.
+/// - `Concrete`: A monomorphic type, no quantifiers. Used during type inference
+///   before generalization.
+/// - `Quantified(n, qt)`: A polymorphic type with `n` bound variables. The inner
+///   `QualifiedType` uses `TVar(Quantified(i))` to reference bound variables.
 ///
-/// Type schemes can be created by calling .quantify() on a qualified
-/// type.
+/// Example: `fn square<D: Dim>(x: D) -> D²` has type scheme
+/// `Quantified(1, QualifiedType { inner: Fn([Q0], Q0²), bounds: [IsDim(Q0)] })`
+/// where `Q0` means `TVar(Quantified(0))`.
+///
+/// Created by calling `.quantify()` on a `QualifiedType`, or `.generalize()` on a `Concrete`.
 #[derive(Clone, Debug, PartialEq, Eq)]
 pub enum TypeScheme {
     Concrete(Type),
@@ -104,6 +109,10 @@ impl TypeScheme {
         }
     }
 
+    pub(crate) fn is_polymorphic(&self) -> bool {
+        matches!(self, TypeScheme::Quantified(n, _) if *n > 0)
+    }
+
     pub(crate) fn to_readable_type(
         &self,
         registry: &crate::dimension::DimensionRegistry,
@@ -123,7 +132,7 @@ impl TypeScheme {
 
             for type_parameter in &type_parameters {
                 markup += m::space();
-                markup += m::type_identifier(type_parameter.unsafe_name().to_string());
+                markup += m::type_identifier(type_parameter.unsafe_name().to_compact_string());
 
                 if instantiated_type.bounds.is_dtype_bound(type_parameter) {
                     markup += m::operator(":");
@@ -219,7 +228,7 @@ impl PrettyPrint for TypeScheme {
                 for type_parameter in &type_parameters {
                     markup += m::keyword("forall");
                     markup += m::space();
-                    markup += m::type_identifier(type_parameter.unsafe_name().to_string());
+                    markup += m::type_identifier(type_parameter.unsafe_name().to_compact_string());
 
                     if instantiated_type.bounds.is_dtype_bound(type_parameter) {
                         markup += m::operator(":");
