@@ -19,6 +19,7 @@ use crate::{
     decorator::Decorator, markup::Markup, number::Number, prefix::Prefix,
     prefix_parser::AcceptsPrefix, pretty_print::PrettyPrint, span::Span,
 };
+use num_traits::{CheckedAdd, CheckedMul};
 
 /// Dimension type
 #[derive(Clone, Debug, PartialEq, Eq)]
@@ -139,6 +140,12 @@ impl DType {
     }
 
     fn canonicalize(&mut self) {
+        self.try_canonicalize()
+            .expect("overflow in dimension type exponent computation");
+    }
+
+    /// Canonicalize with overflow checking. Returns `None` if an overflow occurs.
+    fn try_canonicalize(&mut self) -> Option<()> {
         // Move all type-variable and tgen factors to the front, sort by name
         Arc::make_mut(&mut self.factors).sort_by(|(f1, _), (f2, _)| match (f1, f2) {
             (DTypeFactor::TVar(v1), DTypeFactor::TVar(v2)) => v1.cmp(v2),
@@ -153,12 +160,12 @@ impl DType {
         });
 
         // Merge powers of equal factors:
-        let mut new_factors = Vec::new();
+        let mut new_factors: Vec<DtypeFactorPower> = Vec::new();
         for (f, n) in self.factors.iter() {
             if let Some((last_f, last_n)) = new_factors.last_mut()
                 && f == last_f
             {
-                *last_n += n;
+                *last_n = last_n.checked_add(n)?;
                 continue;
             }
             new_factors.push((f.clone(), *n));
@@ -168,6 +175,14 @@ impl DType {
         new_factors.retain(|(_, n)| *n != Exponent::from_integer(0));
 
         self.factors = Arc::new(new_factors);
+        Some(())
+    }
+
+    /// Like `from_factors`, but returns `None` if an overflow occurs during canonicalization.
+    pub fn try_from_factors(factors: Arc<Vec<DtypeFactorPower>>) -> Option<DType> {
+        let mut dtype = DType { factors };
+        dtype.try_canonicalize()?;
+        Some(dtype)
     }
 
     pub fn multiply(&self, other: &DType) -> DType {
@@ -176,6 +191,13 @@ impl DType {
         DType::from_factors(factors)
     }
 
+    /// Like `multiply`, but returns `None` if an overflow occurs.
+    pub fn try_multiply(&self, other: &DType) -> Option<DType> {
+        let mut factors = self.factors.clone();
+        Arc::make_mut(&mut factors).extend(other.factors.iter().cloned());
+        DType::try_from_factors(factors)
+    }
+
     pub fn power(&self, n: Exponent) -> DType {
         let factors = self
             .factors
@@ -185,6 +207,16 @@ impl DType {
         DType::from_factors(Arc::new(factors))
     }
 
+    /// Like `power`, but returns `None` if the exponent computation overflows.
+    pub fn try_power(&self, n: Exponent) -> Option<DType> {
+        let factors: Option<Vec<_>> = self
+            .factors
+            .iter()
+            .map(|(f, m)| n.checked_mul(m).map(|exp| (f.clone(), exp)))
+            .collect();
+        factors.and_then(|f| DType::try_from_factors(Arc::new(f)))
+    }
+
     pub fn inverse(&self) -> DType {
         self.power(-Exponent::from_integer(1))
     }
@@ -193,6 +225,11 @@ impl DType {
         self.multiply(&other.inverse())
     }
 
+    /// Like `divide`, but returns `None` if an overflow occurs.
+    pub fn try_divide(&self, other: &DType) -> Option<DType> {
+        self.try_multiply(&other.inverse())
+    }
+
     pub fn type_variables(&self, including_type_parameters: bool) -> Vec<TypeVariable> {
         let mut vars: Vec<_> = self
             .factors
@@ -302,6 +339,14 @@ pub struct StructInfo {
     pub fields: IndexMap<CompactString, (Span, Type)>,
 }
 
+/// A monomorphic type (no quantifiers).
+///
+/// - `TVar`: Unification variable, to be solved during type inference. Example: when
+///   type-checking `fn f(x) = x + 1`, the parameter `x` gets a fresh `TVar(Named("T0"))`.
+/// - `TPar`: User-written type parameter in a polymorphic definition. Example: in
+///   `fn square<D: Dim>(x: D) -> D²`, the `D` in type annotations becomes `TPar("D")`.
+///
+/// During generalization, both `TVar` and `TPar` become `TVar(Quantified(i))` in a `TypeScheme`.
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum Type {
     TVar(TypeVariable),
@@ -452,6 +497,36 @@ impl Type {
         matches!(self, Type::Fn(..))
     }
 
+    /// Check if two types have incompatible type constructors (i.e., they can never be unified).
+    /// For example, a Dimension type can never equal a Fn type, even if one contains type variables.
+    /// This is used for early detection of type mismatches in the constraint solver.
+    pub(crate) fn has_incompatible_constructor(&self, other: &Type) -> bool {
+        use Type::*;
+
+        // A Dimension type containing only a type variable can be unified with any type
+        let is_dimension_with_only_tvar = |t: &Type| matches!(t, Dimension(d) if d.deconstruct_as_single_type_variable().is_some());
+
+        match (self, other) {
+            // Type variables and type parameters can potentially match anything
+            (TVar(_), _) | (_, TVar(_)) | (TPar(_), _) | (_, TPar(_)) => false,
+
+            // A Dimension containing only a type variable can match anything
+            (t1, t2) if is_dimension_with_only_tvar(t1) || is_dimension_with_only_tvar(t2) => false,
+
+            // Same constructors might be compatible (need further unification)
+            (Dimension(_), Dimension(_))
+            | (Boolean, Boolean)
+            | (String, String)
+            | (DateTime, DateTime)
+            | (Fn(_, _), Fn(_, _))
+            | (Struct(_), Struct(_))
+            | (List(_), List(_)) => false,
+
+            // Different concrete constructors are incompatible
+            _ => true,
+        }
+    }
+
     pub(crate) fn type_variables(&self, including_type_parameters: bool) -> Vec<TypeVariable> {
         match self {
             Type::TVar(v) => vec![v.clone()],
@@ -1041,6 +1116,7 @@ fn decorator_markup(decorators: &Vec<Decorator>) -> Markup {
             + match decorator {
                 Decorator::MetricPrefixes => m::decorator("@metric_prefixes"),
                 Decorator::BinaryPrefixes => m::decorator("@binary_prefixes"),
+                Decorator::Abbreviation => m::decorator("@abbreviation"),
                 Decorator::Aliases(names) => {
                     m::decorator("@aliases")
                         + m::operator("(")
@@ -1519,6 +1595,32 @@ impl PrettyPrint for Expression<'_> {
             BinaryOperator { op, lhs, rhs, .. } => pretty_print_binop(op, lhs, rhs),
             BinaryOperatorForDate { op, lhs, rhs, .. } => pretty_print_binop(op, lhs, rhs),
             FunctionCall { name, args, .. } => {
+                // Special case: render special temperature conversion functions in their sugar form:
+                if args.len() == 1 {
+                    // from_celsius(x) / from_fahrenheit(x) -> "… °C" / "… °F"
+                    if *name == "from_celsius" {
+                        return with_parens_liberal(&args[0]) + m::space() + m::unit("°C");
+                    } else if *name == "from_fahrenheit" {
+                        return with_parens_liberal(&args[0]) + m::space() + m::unit("°F");
+                    }
+                    // °C(x) / celsius(x) / degree_celsius(x) -> "x -> °C"
+                    // °F(x) / fahrenheit(x) / degree_fahrenheit(x) -> "x -> °F"
+                    if *name == "°C" || *name == "celsius" || *name == "degree_celsius" {
+                        return with_parens_liberal(&args[0])
+                            + m::space()
+                            + m::operator("->")
+                            + m::space()
+                            + m::unit("°C");
+                    } else if *name == "°F" || *name == "fahrenheit" || *name == "degree_fahrenheit"
+                    {
+                        return with_parens_liberal(&args[0])
+                            + m::space()
+                            + m::operator("->")
+                            + m::space()
+                            + m::unit("°F");
+                    }
+                }
+
                 m::identifier(name.to_compact_string())
                     + m::operator("(")
                     + itertools::Itertools::intersperse(
@@ -1533,6 +1635,26 @@ impl PrettyPrint for Expression<'_> {
                 args,
                 ..
             } => {
+                // See above
+                if args.len() == 1
+                    && let Expression::Identifier { name, .. } = expr.as_ref()
+                {
+                    if *name == "°C" || *name == "celsius" || *name == "degree_celsius" {
+                        return with_parens_liberal(&args[0])
+                            + m::space()
+                            + m::operator("->")
+                            + m::space()
+                            + m::unit("°C");
+                    } else if *name == "°F" || *name == "fahrenheit" || *name == "degree_fahrenheit"
+                    {
+                        return with_parens_liberal(&args[0])
+                            + m::space()
+                            + m::operator("->")
+                            + m::space()
+                            + m::unit("°F");
+                    }
+                }
+
                 expr.pretty_print()
                     + m::operator("(")
                     + itertools::Itertools::intersperse(
