@@ -269,6 +269,15 @@ impl Unit {
         Self::from_factors(factors)
     }
 
+    /// Returns a new unit with all prefixes stripped from the unit factors.
+    pub fn without_prefixes(&self) -> Self {
+        Self::from_factors(self.iter().map(|factor| UnitFactor {
+            prefix: Prefix::none(),
+            unit_id: factor.unit_id.clone(),
+            exponent: factor.exponent,
+        }))
+    }
+
     pub fn to_base_unit_representation(&self) -> (Self, ConversionFactor) {
         // TODO: reduce wrapping/unwrapping
         let mut base_unit_representation = Product::unity();
@@ -293,6 +302,20 @@ impl Unit {
         (base_unit_representation, factor)
     }
 
+    /// Returns the smaller of the two units, i.e. the one with the smaller
+    /// conversion factor to the base unit representation. This is useful
+    /// to ensure commutativity of addition/subtraction.
+    pub fn smaller_unit<'a>(&'a self, other: &'a Self) -> &'a Self {
+        let (_, self_factor) = self.to_base_unit_representation();
+        let (_, other_factor) = other.to_base_unit_representation();
+
+        if self_factor.to_f64() <= other_factor.to_f64() {
+            self
+        } else {
+            other
+        }
+    }
+
     #[cfg(test)]
     pub fn meter() -> Self {
         Self::new_base(
