@@ -6,10 +6,11 @@ use compact_str::{CompactString, ToCompactString};
 use indexmap::IndexMap;
 use num_traits::ToPrimitive;
 
-use crate::dimension::DimensionRegistry;
 use crate::interpreter::RuntimeErrorKind;
 use crate::list::NumbatList;
+use crate::prefix_transformer::Transformer;
 use crate::span::Span;
+use crate::typechecker::TypeChecker;
 use crate::typechecker::type_scheme::TypeScheme;
 use crate::typed_ast::StructInfo;
 use crate::{
@@ -19,6 +20,7 @@ use crate::{
     math,
     number::Number,
     prefix::Prefix,
+    pretty_print::FormatOptions,
     quantity::{Quantity, QuantityError},
     unit::Unit,
     unit_registry::{UnitMetadata, UnitRegistry},
@@ -271,7 +273,9 @@ impl CallFrame {
 
 pub struct ExecutionContext<'a> {
     pub print_fn: &'a mut PrintFunction,
-    pub dimension_registry: &'a DimensionRegistry,
+    pub unit_name_to_constant_idx: &'a HashMap<CompactString, u16>,
+    pub prefix_transformer: &'a Transformer,
+    pub typechecker: &'a TypeChecker,
 }
 
 /// Metadata for a single FFI call argument
@@ -281,6 +285,14 @@ pub struct FfiCallArg {
     pub type_: TypeScheme,
 }
 
+/// Metadata for an FFI call (arguments and return type)
+#[derive(Clone)]
+pub struct FfiCallArgs {
+    pub args: Vec<FfiCallArg>,
+    /// Return type for FFI functions. None for procedures.
+    pub return_type: Option<TypeScheme>,
+}
+
 #[derive(Clone)]
 pub struct Vm {
     /// The actual code and spans of the program, structured by function name. The code
@@ -314,8 +326,8 @@ pub struct Vm {
     /// List of registered native/foreign functions
     ffi_callables: IndexMap<&'static str, &'static ForeignFunction>,
 
-    /// Metadata for FFI call arguments (spans and types).
-    ffi_call_args: Vec<FfiCallArg>,
+    /// Metadata for FFI calls (argument spans/types and return type).
+    ffi_call_args: Vec<FfiCallArgs>,
 
     /// The call stack
     frames: Vec<CallFrame>,
@@ -511,11 +523,28 @@ impl Vm {
         Some(position as u16)
     }
 
-    pub(crate) fn add_ffi_call_args(&mut self, args: Vec<FfiCallArg>) -> u16 {
-        let start_idx = self.ffi_call_args.len();
-        self.ffi_call_args.extend(args);
+    /// Simplify a quantity using the unit registry and constants.
+    pub fn simplify_quantity(
+        &self,
+        q: &Quantity,
+        unit_name_to_constant_idx: &HashMap<CompactString, u16>,
+    ) -> Quantity {
+        q.full_simplify_with_registry(&self.unit_registry, |name| {
+            unit_name_to_constant_idx
+                .get(name)
+                .and_then(|idx| self.constants.get(*idx as usize))
+                .and_then(|constant| match constant {
+                    Constant::Unit(u) => Some(u.clone()),
+                    _ => None,
+                })
+        })
+    }
+
+    pub(crate) fn add_ffi_call_args(&mut self, call_args: FfiCallArgs) -> u16 {
+        let idx = self.ffi_call_args.len();
+        self.ffi_call_args.push(call_args);
         assert!(self.ffi_call_args.len() <= u16::MAX as usize);
-        start_idx as u16
+        idx as u16
     }
 
     pub fn disassemble(&self) {
@@ -737,8 +766,12 @@ impl Vm {
                             .map_err(|e| self.runtime_error(RuntimeErrorKind::QuantityError(e)))?
                             .ok_or_else(|| self.runtime_error(RuntimeErrorKind::DivisionByZero))?),
                         // If the user specifically converted the type of a unit, we should NOT simplify this value
-                        // before any operations are applied to it
-                        Op::ConvertTo => lhs.convert_to(rhs.unit()).map(Quantity::no_simplify),
+                        // before any operations are applied to it.
+                        // Also, preserve the RHS for display purposes (e.g., `6 hours -> 45 min` should
+                        // display as `8 × 45 min` instead of `360 min`).
+                        Op::ConvertTo => lhs
+                            .convert_to(rhs.unit())
+                            .map(|q| q.no_simplify().with_conversion_target(rhs)),
                         _ => unreachable!(),
                     };
                     self.push_quantity(
@@ -895,16 +928,15 @@ impl Vm {
                 Op::FFICallFunction | Op::FFICallProcedure => {
                     let function_idx = self.read_u16() as usize;
                     let num_args = self.read_u16() as usize;
-                    let call_args_start = self.read_u16() as usize;
+                    let call_args_idx = self.read_u16() as usize;
                     let foreign_function = &self.ffi_callables[function_idx];
 
                     debug_assert!(foreign_function.arity.contains(&num_args));
 
-                    let call_args =
-                        self.ffi_call_args[call_args_start..call_args_start + num_args].to_vec();
+                    let ffi_call_args = self.ffi_call_args[call_args_idx].clone();
                     let mut args = VecDeque::new();
                     for i in 0..num_args {
-                        let call_arg = &call_args[num_args - 1 - i];
+                        let call_arg = &ffi_call_args.args[num_args - 1 - i];
                         args.push_front(Arg {
                             value: self.pop(),
                             type_: call_arg.type_.clone(),
@@ -912,10 +944,27 @@ impl Vm {
                         });
                     }
 
+                    // For the print procedure, simplify quantity arguments before printing
+                    let (proc_name, _) = self.ffi_callables.get_index(function_idx).unwrap();
+                    if *proc_name == "print" {
+                        for arg in args.iter_mut() {
+                            if let Value::Quantity(q) = &arg.value {
+                                let simplified =
+                                    self.simplify_quantity(q, ctx.unit_name_to_constant_idx);
+                                arg.value = Value::Quantity(simplified);
+                            }
+                        }
+                    }
+
                     match &self.ffi_callables[function_idx].callable {
                         Callable::Function(function) => {
-                            let result =
-                                (function)(ctx, args).map_err(|e| self.runtime_error(*e))?;
+                            let return_type = ffi_call_args
+                                .return_type
+                                .as_ref()
+                                .expect("FFI functions must have a return type");
+                            let mut ffi_ctx = ffi::FfiContext::new(ctx, &self.constants);
+                            let result = (function)(&mut ffi_ctx, args, return_type)
+                                .map_err(|e| self.runtime_error(*e))?;
                             self.push(result);
                         }
                         Callable::Procedure(procedure) => {
@@ -932,7 +981,7 @@ impl Vm {
                 }
                 Op::CallCallable => {
                     let num_args = self.read_u16() as usize;
-                    let call_args_start = self.read_u16() as usize;
+                    let call_args_idx = self.read_u16() as usize;
 
                     let callable = self.pop();
                     match callable.unsafe_as_function_reference() {
@@ -952,12 +1001,10 @@ impl Vm {
                                 .expect("Foreign function exists")
                                 as usize;
 
-                            let call_args = self.ffi_call_args
-                                [call_args_start..call_args_start + num_args]
-                                .to_vec();
+                            let ffi_call_args = self.ffi_call_args[call_args_idx].clone();
                             let mut args = VecDeque::new();
                             for i in 0..num_args {
-                                let call_arg = &call_args[num_args - 1 - i];
+                                let call_arg = &ffi_call_args.args[num_args - 1 - i];
                                 args.push_front(Arg {
                                     value: self.pop(),
                                     type_: call_arg.type_.clone(),
@@ -967,7 +1014,12 @@ impl Vm {
 
                             match &self.ffi_callables[function_idx].callable {
                                 Callable::Function(function) => {
-                                    let result = (function)(ctx, args)
+                                    let return_type = ffi_call_args
+                                        .return_type
+                                        .as_ref()
+                                        .expect("FFI functions must have a return type");
+                                    let mut ffi_ctx = ffi::FfiContext::new(ctx, &self.constants);
+                                    let result = (function)(&mut ffi_ctx, args, return_type)
                                         .map_err(|e| self.runtime_error(*e))?;
                                     self.push(result);
                                 }
@@ -1001,11 +1053,15 @@ impl Vm {
                 Op::JoinString => {
                     let num_parts = self.read_u16() as usize;
                     let mut joined = CompactString::with_capacity(num_parts);
-                    let to_str = |value| match value {
-                        Value::Quantity(q) => q.full_simplify().to_compact_string(),
+                    let to_str = |value, vm: &Self, ctx: &ExecutionContext| match value {
+                        Value::Quantity(q) => vm
+                            .simplify_quantity(&q, ctx.unit_name_to_constant_idx)
+                            .to_compact_string(),
                         Value::Boolean(b) => b.to_compact_string(),
                         Value::String(s) => s.to_compact_string(),
-                        Value::DateTime(dt) => crate::datetime::to_string(&dt),
+                        Value::DateTime(dt) => {
+                            crate::datetime::to_string(&dt, &FormatOptions::default())
+                        }
                         Value::FunctionReference(r) => r.to_compact_string(),
                         s @ Value::StructInstance(..) => s.to_compact_string(),
                         l @ Value::List(_) => l.to_compact_string(),
@@ -1026,7 +1082,8 @@ impl Vm {
                         let part = match self.pop() {
                             Value::FormatSpecifiers(Some(specifiers)) => match self.pop() {
                                 Value::Quantity(q) => {
-                                    let q = q.full_simplify();
+                                    let q =
+                                        self.simplify_quantity(&q, ctx.unit_name_to_constant_idx);
 
                                     let mut vars = HashMap::new();
                                     vars.insert(
@@ -1052,15 +1109,18 @@ impl Vm {
                                 }
                                 value => {
                                     let mut vars = HashMap::new();
-                                    vars.insert("value".to_owned(), to_str(value).to_string());
+                                    vars.insert(
+                                        "value".to_owned(),
+                                        to_str(value, self, ctx).to_string(),
+                                    );
 
                                     strfmt::strfmt(&format!("{{value{specifiers}}}"), &vars)
                                         .map(CompactString::from)
                                         .map_err(|e| map_strfmt_error_to_runtime_error(self, e))?
                                 }
                             },
-                            Value::FormatSpecifiers(None) => to_str(self.pop()),
-                            v => to_str(v),
+                            Value::FormatSpecifiers(None) => to_str(self.pop(), self, ctx),
+                            v => to_str(v, self, ctx),
                         };
                         joined = part + &joined; // reverse order
                     }
@@ -1176,10 +1236,14 @@ fn vm_basic() {
     vm.add_op(Op::Return, Span::dummy());
 
     let mut print_fn = |_: &Markup| {};
-    let dimension_registry = DimensionRegistry::default();
+    let unit_name_to_constant_idx = HashMap::new();
+    let prefix_transformer = Transformer::new();
+    let typechecker = TypeChecker::default();
     let mut ctx = ExecutionContext {
         print_fn: &mut print_fn,
-        dimension_registry: &dimension_registry,
+        unit_name_to_constant_idx: &unit_name_to_constant_idx,
+        prefix_transformer: &prefix_transformer,
+        typechecker: &typechecker,
     };
 
     assert_eq!(
