@@ -266,6 +266,29 @@ fn test_conversions() {
     expect_output_with_context(&mut ctx, "12 deg -> x", "12°");
 }
 
+#[test]
+fn test_conversions_with_nontrivial_magnitude() {
+    expect_output("6 hours -> 45 min", "8 × 45 min");
+    expect_output("1 KiB -> 16 B", "64 × 16 B");
+    expect_output("10 m -> 2 m", "5 × 2 m");
+    expect_output("2 m -> 10 m", "0.2 × 10 m");
+    expect_output("10 m -> 0.5 m", "20 × 0.5 m");
+    expect_output("10 m -> (m/3)", "30 × 0.333333 m");
+    expect_output("1/(30 mpg) -> L/(100 km)", "7.84049 × 0.01 l/km");
+
+    // When RHS value is 1 (explicitly or implicitly), behave as usual
+    expect_output("10 m -> m", "10 m");
+    expect_output("10 m -> 1 m", "10 m");
+
+    // After arithmetic operations, the multiplier should be cleared
+    expect_output("(6 hours -> 45 min) + 1 min", "361 min");
+    expect_output("(1 KiB -> 16 B) + 1 B", "1025 B");
+
+    // Inverse units also use the multiplication format
+    expect_output("3 / m -> 0.2 / m", "15 × 0.2 m⁻¹");
+    expect_output("10 / s -> 0.5 / s", "20 × 0.5 s⁻¹");
+}
+
 #[test]
 fn test_implicit_conversion() {
     let mut ctx = get_test_context();
@@ -361,7 +384,7 @@ fn test_math() {
     expect_output("atan2(100 cm, 1 m) / (pi / 4)", "1");
     expect_failure(
         "atan2(100 cm, 1 m²)",
-        "Could not solve the following constraints",
+        "Could not solve the following constraint",
     );
 
     expect_output("mod(5, 3)", "2");
@@ -369,7 +392,7 @@ fn test_math() {
     expect_output("mod(8 cm, 5 cm)", "3 cm");
     expect_output("mod(235 cm, 1 m)", "35 cm");
     expect_output("mod(2 m, 7 cm)", "0.04 m");
-    expect_failure("mod(8 m, 5 s)", "Could not solve the following constraints")
+    expect_failure("mod(8 m, 5 s)", "Could not solve the following constraint")
 }
 
 #[test]
@@ -463,16 +486,16 @@ fn test_incompatible_dimension_errors() {
 
 #[test]
 fn test_temperature_conversions() {
-    expect_output("from_celsius(11.5)", "284.65 K");
-    expect_output("from_fahrenheit(89.3)", "304.983 K");
-    expect_output("0 K -> celsius", "-273.15");
+    expect_output("11.5 °C", "284.65 K");
+    expect_output("89.3 °F", "304.983 K");
+    expect_output("0 K -> °C", "-273.15");
     expect_output("fahrenheit(30 K)", "-405.67");
-    expect_output("from_celsius(100) -> celsius", "100");
-    expect_output("from_fahrenheit(100) -> fahrenheit", "100.0");
-    expect_output("from_celsius(123 K -> celsius)", "123 K");
-    expect_output("from_fahrenheit(123 K -> fahrenheit)", "123 K");
+    expect_output("100 °C -> °C", "100");
+    expect_output("100 °F -> °F", "100.0");
+    expect_output("(123 K -> °C) °C", "123 K");
+    expect_output("(123 K -> °F) °F", "123 K");
 
-    expect_output("-40 -> from_fahrenheit -> celsius", "-40");
+    expect_output("-40 °F -> °C", "-40");
 }
 
 #[test]
@@ -508,16 +531,46 @@ fn test_last_result_identifier() {
     expect_output_with_context(&mut ctx, "_", "3");
 }
 
+#[test]
+fn test_addition_subtraction_commutativity() {
+    // Addition uses the smaller of the two operand units to ensure commutativity
+    expect_output("2 min + 30 s", "150 s");
+    expect_output("30 s + 2 min", "150 s");
+    expect_output("1 m + 50 cm", "150 cm");
+    expect_output("50 cm + 1 m", "150 cm");
+
+    // Similar for subtraction to ensure anticommutativity
+    expect_output("2 min - 30 s", "90 s");
+    expect_output("30 s - 2 min", "-90 s");
+    expect_output("1 m - 50 cm", "50 cm");
+    expect_output("50 cm - 1 m", "-50 cm");
+
+    // Test with three operands
+    expect_output("1 hour + 30 min + 45 s", "5445 s");
+    expect_output("1 hour + 45 s + 30 min", "5445 s");
+    expect_output("30 min + 1 hour + 45 s", "5445 s");
+    expect_output("30 min + 45 s + 1 hour", "5445 s");
+    expect_output("45 s + 1 hour + 30 min", "5445 s");
+    expect_output("45 s + 30 min + 1 hour", "5445 s");
+}
+
+// Regression test for https://github.com/sharkdp/numbat/issues/589
+#[test]
+fn test_issue_589_floor_in_precision() {
+    expect_output("1.5 weeks - floor_in(days, 1.5 weeks)", "0.5 day");
+    expect_output("1.5 weeks - floor_in(days, 1.5 weeks) -> hours", "12 h");
+}
+
 #[test]
 fn test_misc_examples() {
     expect_output("1920/16*9", "1080");
     expect_output("2^32", "4_294_967_296");
     expect_output("sqrt(1.4^2 + 1.5^2) * cos(pi/3)^2", "0.512957");
 
-    expect_output("2min + 30s", "2.5 min");
-    expect_output("2min + 30s -> sec", "150 s");
+    expect_output("2min + 30s", "150 s");
+    expect_output("30s + 2min", "150 s");
     expect_output("4/3 * pi * (6000km)³", "9.04779e+11 km³");
-    expect_output("40kg * 9.8m/s^2 * 150cm", "588 kg·m²/s²");
+    expect_output("40kg * 9.8m/s^2 * 150cm", "588 J");
     expect_output("sin(30°)", "0.5");
 
     expect_output("60mph -> m/s", "26.8224 m/s");
@@ -557,9 +610,31 @@ fn test_full_simplify() {
         "500 cm/m",
     );
 
-    expect_output("1 Wh/W", "1 Wh/W"); // This output is not great (and should be improved). But we keep this as a regression test for a bug in previous versions.
+    expect_output("1 Wh/W", "1 h");
 
     expect_output("1 × (m/s)^2/(m/s)", "1 m/s");
+
+    expect_output("mph * s/m", "0.44704");
+    expect_output("3% * 1kg", "0.03 kg");
+    expect_output("m * g / cm", "100 g");
+    expect_output("gal/in", "231 in²");
+    expect_output(
+        "4 pi ε0 ℏ^2 / (electron_charge^2 electron_mass)",
+        "5.29177e-11 m",
+    );
+
+    expect_output("J/s", "1 W");
+    expect_output("Pa m²", "1 N");
+    expect_output("50 Ω * 2 A", "100 V");
+    expect_output("10 N / 5 Pa", "2 m²");
+    expect_output(
+        "sqrt(ℏ gravitational_constant / speed_of_light^3)",
+        "1.61626e-35 m",
+    );
+
+    expect_output("1 dpi * 1 in", "1 dot");
+    expect_output("1 mph * 1 hour", "1 mi");
+    expect_output("1 dot / 96 dpi", "0.0104167 in");
 }
 
 #[test]
@@ -597,6 +672,31 @@ fn test_name_clash_errors() {
     expect_failure("fn _()=0", "Reserved identifier");
 }
 
+#[test]
+fn test_parameter_shadowing() {
+    // Function parameters can shadow unit names.
+    // 'h' shadows the unit alias for 'hour'
+    expect_output(
+        "fn area(h: Length, w: Length) -> Area = h × w
+         area(3 m, 4 m)",
+        "12 m²",
+    );
+
+    // Local variables can also shadow unit names.
+    expect_output(
+        "fn test(x: Scalar) -> Scalar = h where h = x + 1
+         test(2)",
+        "3",
+    );
+
+    // This also works for constants (`e` for Euler's number in this case).
+    expect_output(
+        "fn f(e: Scalar) -> Scalar = e + 1
+         f(2)",
+        "3",
+    );
+}
+
 #[test]
 fn test_type_check_errors() {
     expect_failure("foo", "Unknown identifier 'foo'");
@@ -806,6 +906,13 @@ fn test_string_interpolation() {
         ",
         "Incorrect type for format specifiers: Unknown format code Some('f') for object of type 'str'",
     );
+
+    // Units should be simplified in string interpolation
+    expect_output(
+        "let power = 1 J/s
+        \"{power}\"",
+        "\"1 W\"",
+    );
 }
 
 #[test]
@@ -1011,6 +1118,117 @@ fn test_statement_pretty_printing() {
         "fn f<A>(x: A) -> A = y\n  where y: A = x",
     );
 }
+
+#[test]
+fn test_parse() {
+    // Parsing valid numbers as Scalar
+    expect_output("let x: Scalar = parse(\"3.14\"); x", "3.14");
+    expect_output("let x: Scalar = parse(\"  42  \"); x", "42");
+    expect_output("let x: Scalar = parse(\"-1.5\"); x", "-1.5");
+    expect_output("let x: Scalar = parse(\"1e10\"); x", "10_000_000_000");
+    expect_output("let x: Scalar = parse(\"-2.5e-3\"); x", "-0.0025");
+    expect_output("let x: Scalar = parse(\"0xFF\"); x", "255");
+    expect_output("let x: Scalar = parse(\"0b1010\"); x", "10");
+    expect_output("let x: Scalar = parse(\"NaN\"); x", "NaN");
+    expect_output("let x: Scalar = parse(\"inf\"); x", "inf");
+    expect_output("let x: Scalar = parse(\"-inf\"); x", "-inf");
+
+    // Parsing quantities with units
+    expect_output("let x: Length = parse(\"1.5 km\"); x", "1.5 km");
+    expect_output("let x: Length = parse(\"100 m\"); x", "100 m");
+    expect_output("let x: Length = parse(\"-50 cm\"); x", "-50 cm");
+    expect_output("let x: Mass = parse(\"2.5 kg\"); x", "2.5 kg");
+    expect_output("let x: Length = parse(\"inf m\"); x", "inf m");
+    expect_output("let x: Length = parse(\"NaN m\"); x", "NaN m");
+
+    // Error: invalid format
+    expect_failure("let x: Scalar = parse(\"not a number\")", "Invalid pattern");
+    expect_failure("let x: Scalar = parse(\"1 1\")", "Invalid pattern");
+
+    // Error: unknown unit
+    expect_failure(
+        "let x: Length = parse(\"1 unknown_unit\")",
+        "Expected unit identifier",
+    );
+
+    // Error: polymorphic return type (no type annotation)
+    expect_failure(
+        "parse(\"1.5\")",
+        "parse() requires a type annotation for the return type",
+    );
+
+    // Error: type mismatch (parsed Mass but expected Length)
+    expect_failure(
+        "let x: Length = parse(\"1 kg\")",
+        "Type mismatch: expected `Length`, got `Mass`",
+    );
+    expect_failure(
+        "let x: Scalar = parse(\"1 kg\")",
+        "Type mismatch: expected `Scalar`, got `Mass`",
+    );
+    expect_failure(
+        "let x: Length = parse(\"1\")",
+        "Type mismatch: expected `Length`, got `Scalar`",
+    );
+
+    // Zero is compatible with any type
+    expect_output("let x: Length = parse(\"0\"); x", "0");
+    expect_output("let x: Mass = parse(\"0\"); x", "0");
+    expect_output("let x: Length = parse(\"0 kg\"); x", "0");
+
+    // User-defined base unit should be available immediately
+    expect_output(
+        "unit jump: Length\nlet x: Length = parse(\"4 jump\"); x",
+        "4 jump",
+    );
+
+    // User-defined derived unit should be available immediately
+    expect_output(
+        "unit leap = 2 meter\nlet x: Length = parse(\"3 leap\"); x",
+        "3 leap",
+    );
+}
+
+#[test]
+fn test_temperature_syntactic_sugar() {
+    // Entering temperatures in °C/°F
+    expect_output("0 °C", "273.15 K");
+    expect_output("100 °C", "373.15 K");
+    expect_output("-5 °C", "268.15 K");
+    expect_output("32 °F", "273.15 K");
+    expect_output("212 °F", "373.15 K");
+
+    // Conversions to °C/°F
+    expect_output("273.15 K -> °C", "0");
+    expect_output("373.15 K -> °F", "212");
+
+    // Conversions between °C and °F
+    expect_output("100 °C -> °F", "212");
+    expect_output("212 °F -> °C", "100.0");
+    expect_output("-40 °C -> °F", "-40.0");
+    expect_output("-40 °F -> °C", "-40");
+
+    // Expressions
+    expect_output("(50 + 50) °C", "373.15 K");
+    expect_output("0 °C + 10 K -> °C", "10");
+
+    // Aliases work too
+    expect_output("20 celsius", "293.15 K");
+    expect_output("20 degree_celsius", "293.15 K");
+    expect_output("68 fahrenheit", "293.15 K");
+    expect_output("68 degree_fahrenheit", "293.15 K");
+
+    // Pretty-printing renders using °C/°F syntax
+    expect_pretty_print("let t = 20 °C", "let t: Temperature = 20 °C");
+    expect_pretty_print("let t = -5 °F", "let t: Temperature = (-5) °F");
+    expect_pretty_print("let v = 300 K -> °C", "let v: Scalar = 300 kelvin -> °C");
+    expect_pretty_print("let v = 300 K -> °F", "let v: Scalar = 300 kelvin -> °F");
+
+    // Aliases pretty-print as °C/°F
+    expect_pretty_print("let t = 20 celsius", "let t: Temperature = 20 °C");
+    expect_pretty_print("let t = 68 fahrenheit", "let t: Temperature = 68 °F");
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -1115,7 +1333,7 @@ mod tests {
         fn issue505_angles() {
             insta::assert_snapshot!(fail("assert_eq(-77° + 0′ + 32″, -77.0089°, 1e-4°)"), @r###"
             Assertion failed because the following two quantities differ by 0.0178°, which is more than 0.0001°:
-              -76.9911°
+              -76.9911° (-277_168″)
               -77.0089°
             "###);
         }
